/***************************************************************************

Jump Coaster memory map (preliminary)
it runs on hardware very similar to Scramble, but graphics are 3bpp instead
of 2bpp. The palette has 256 colors, as opposed to the 32 of the Scramble
hardware. However the color codes seem to be limited to 3 bits like in
Scramble, so there might be a palette bank selector register.
I think f002-f003 together specify the palatte. They're set to a
different combination of 00/ff at the beginning of each level. I tried
implementing it, and it looks OK, but not quite right. The ape at the
top of the screen is still yellow, instead of brown as one would expect.


0000-7fff ROM
c000-c7ff RAM
d000-d03f Color/Scroll RAM (2 bytes for each line)
d040-d05f Sprite RAM
d800-dbff Video RAM
dc00-dfff mirror address for the above

read:
c803 (? Only read on startup, but value is not used)
c806 (? Might be some kind of reset/tilt switch )
c900 Protection (?) Checked at 010C
e800 DIP Switches
e802 Input port 1
e803 Input port 2
f800 Watchdog reset (?) (If it is, MAME's default trigger time is too low)

write:
c802 (? Initialized to 2 on startup, not touched again)
c900 (? Initialized to 0 on startup, not touched again)
e000 (? Always 0)
f000 (? Initialized to 0 on startup, not touched again)
f001 NMI enable
f002 (? \ These might be significant, don't know what they do)
f003 (? /						    )
f004 (? Always 0)
f005 (? Always 0)
f006 (? Initialized to 0 on startup, not touched again)
f007 (? Initialized to 0 on startup, not touched again)
f116 \ Screen X/Y flip (don't know which is which since they are
f117 /		  set to the same value, only need to emulate one)
f800 AY8910 Control Port
f801 AY8910 Write Port


***************************************************************************/

#include "driver.h"
#include "vidhrdw/generic.h"



extern unsigned char *galaxian_attributesram;
extern unsigned char *galaxian_bulletsram;
extern int galaxian_bulletsram_size;

void jumpcoas_vh_convert_color_prom(unsigned char *palette, unsigned char *colortable,const unsigned char *color_prom);
void galaxian_flipx_w(int offset,int data);
void galaxian_flipy_w(int offset,int data);
void galaxian_attributes_w(int offset,int data);
int galaxian_vh_start(void);
void galaxian_vh_screenrefresh(struct osd_bitmap *bitmap);


static int protection_r(int offset)
{
     return 0x63;
}


static struct MemoryReadAddress readmem[] =
{
	{ 0x0000, 0x7fff, MRA_ROM },
	{ 0xc000, 0xc7ff, MRA_RAM },
	{ 0xc803, 0xc803, MRA_NOP },
	{ 0xc806, 0xc806, MRA_NOP },
	{ 0xc900, 0xc900, protection_r },
	{ 0xd000, 0xd3ff, MRA_RAM },
	{ 0xd800, 0xdbff, MRA_RAM },
	{ 0xe800, 0xe800, input_port_0_r },
	{ 0xe802, 0xe802, input_port_1_r },
	{ 0xe802, 0xe803, input_port_2_r },
	{ 0xf800, 0xf800, MRA_NOP },
	{ -1 }  /* end of table */
};

static struct MemoryWriteAddress writemem[] =
{
	{ 0x0000, 0x7fff, MWA_ROM },
	{ 0xc000, 0xc7ff, MWA_RAM },
	{ 0xc802, 0xc802, MWA_NOP },
	{ 0xc900, 0xc900, MWA_NOP },
	{ 0xd000, 0xd03f, galaxian_attributes_w, &galaxian_attributesram },
	{ 0xd040, 0xd05f, MWA_RAM, &spriteram, &spriteram_size },
	{ 0xd060, 0xd07f, MWA_RAM, &galaxian_bulletsram, &galaxian_bulletsram_size },
//	{ 0xd080, 0xd3ff, MWA_RAM },
	{ 0xd800, 0xdbff, videoram_w, &videoram, &videoram_size },
	{ 0xdc00, 0xdfff, videoram_w },	/* mirror address, used in the name entry screen */
	{ 0xe000, 0xe000, MWA_NOP },
	{ 0xf000, 0xf000, MWA_NOP },
	{ 0xf001, 0xf001, interrupt_enable_w },
	{ 0xf002, 0xf007, MWA_NOP },
	{ 0xf116, 0xf116, galaxian_flipx_w },
	{ 0xf117, 0xf117, galaxian_flipy_w },
	{ 0xf800, 0xf800, AY8910_control_port_0_w },
	{ 0xf801, 0xf801, AY8910_write_port_0_w },
	{ -1 }  /* end of table */
};



INPUT_PORTS_START( input_ports )
	PORT_START      /* DSW 0 */
	PORT_DIPNAME( 0x03, 0x00, "Coin A", IP_KEY_NONE )
	PORT_DIPSETTING(    0x03, "6 Coins/1 Credit" )
	PORT_DIPSETTING(    0x02, "2 Coins/1 Credit" )
	PORT_DIPSETTING(    0x00, "1 Coin/1 Credit" )
	PORT_DIPSETTING(    0x01, "1 Coin/2 Credits" )
	PORT_DIPNAME( 0x0c, 0x00, "Coin B", IP_KEY_NONE )
	PORT_DIPSETTING(    0x0c, "6 Coins/1 Credit" )
	PORT_DIPSETTING(    0x08, "2 Coins/1 Credit" )
	PORT_DIPSETTING(    0x00, "1 Coin/1 Credit" )
	PORT_DIPSETTING(    0x04, "1 Coin/2 Credits" )
	PORT_DIPNAME( 0x30, 0x00, "Lives", IP_KEY_NONE )
	PORT_DIPSETTING(    0x00, "3" )
	PORT_DIPSETTING(    0x10, "5" )
	PORT_DIPSETTING(    0x20, "7" )
	PORT_BITX( 0,       0x30, IPT_DIPSWITCH_SETTING | IPF_CHEAT, "255", IP_KEY_NONE, IP_JOY_NONE, 0 )
	PORT_DIPNAME( 0x40, 0x00, "Unknown", IP_KEY_NONE )
	PORT_DIPSETTING(    0x00, "Off" )
	PORT_DIPSETTING(    0x40, "On" )
	PORT_DIPNAME( 0x80, 0x00, "Cabinet", IP_KEY_NONE )
	PORT_DIPSETTING(    0x00, "Upright" )
	PORT_DIPSETTING(    0x80, "Cocktail" )

	PORT_START      /* IN1 */
	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_COIN1 )
	PORT_BIT( 0x02, IP_ACTIVE_HIGH, IPT_COIN2 )
	PORT_BIT( 0x04, IP_ACTIVE_HIGH, IPT_UNKNOWN )
	PORT_BIT( 0x08, IP_ACTIVE_HIGH, IPT_START1 )
	PORT_BIT( 0x10, IP_ACTIVE_HIGH, IPT_START2 )
	PORT_BIT( 0x20, IP_ACTIVE_HIGH, IPT_UNKNOWN )
	PORT_BIT( 0x40, IP_ACTIVE_HIGH, IPT_BUTTON1 )
	PORT_BIT( 0x80, IP_ACTIVE_HIGH, IPT_BUTTON1 | IPF_COCKTAIL )

	PORT_START      /* IN2 */
	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_JOYSTICK_LEFT | IPF_8WAY )
	PORT_BIT( 0x02, IP_ACTIVE_HIGH, IPT_JOYSTICK_RIGHT | IPF_8WAY )
	PORT_BIT( 0x04, IP_ACTIVE_HIGH, IPT_JOYSTICK_LEFT | IPF_8WAY | IPF_COCKTAIL )
	PORT_BIT( 0x08, IP_ACTIVE_HIGH, IPT_JOYSTICK_RIGHT | IPF_8WAY | IPF_COCKTAIL )
	PORT_BIT( 0x10, IP_ACTIVE_HIGH, IPT_JOYSTICK_UP | IPF_8WAY )
	PORT_BIT( 0x20, IP_ACTIVE_HIGH, IPT_JOYSTICK_DOWN | IPF_8WAY )
	PORT_BIT( 0x40, IP_ACTIVE_HIGH, IPT_JOYSTICK_UP | IPF_8WAY | IPF_COCKTAIL )
	PORT_BIT( 0x80, IP_ACTIVE_HIGH, IPT_JOYSTICK_DOWN | IPF_8WAY | IPF_COCKTAIL )
INPUT_PORTS_END



static struct GfxLayout charlayout =
{
	8,8,    /* 8*8 characters */
	256,    /* 256 characters */
	3,      /* 3 bits per pixel */
	{ 0, 0x1000*8, 0x2000*8 }, /* the three bitplanes are separated */
	{ 0, 1, 2, 3, 4, 5, 6, 7 },
	{ 0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 6*8, 7*8 },
	8*8     /* every char takes 8 consecutive bytes */
};

static struct GfxLayout spritelayout =
{
	16,16,  /* 16*16 sprites */
	64,    /* 64 sprites */
	3,      /* 3 bits per pixel */
	{ 0, 0x1000*8, 0x2000*8 }, /* the three bitplanes are separated */
	{ 8*8+7, 8*8+6, 8*8+5, 8*8+4, 8*8+3, 8*8+2, 8*8+1, 8*8+0,
			7, 6, 5, 4, 3, 2, 1, 0 },
	{ 0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 6*8, 7*8,
			16*8, 17*8, 18*8, 19*8, 20*8, 21*8, 22*8, 23*8 },
	32*8     /* every char takes 32 consecutive bytes */
};

/* bullets are actually not used by this game, they might not be in the hardware, */
/* but since we draw using the Galaxian video driver we have to provide this definition. */
static struct GfxLayout bulletlayout =
{
	/* there is no gfx ROM for this one, it is generated by the hardware */
	3,1,	/* 3*1 line */
	1,	/* just one */
	1,	/* 1 bit per pixel */
	{ 0 },
	{ 2, 2, 2 },	/* I "know" that this bit of the */
	{ 0 },			/* graphics ROMs is 1 */
	0	/* no use */
};

static struct GfxDecodeInfo gfxdecodeinfo[] =
{
	{ 1, 0x0000, &charlayout,     0, 32 },
	{ 1, 0x0800, &spritelayout,   0, 32 },
	{ 1, 0x0000, &bulletlayout,   0, 1 },	/* actually not used but must be here */
	{ -1 } /* end of array */
};


/* I'm not sure how to colors are actually used. The banana and the apple on
   the title screen uses color 5 and 4, respectively. I assume they should be
   yellow and red, but the way I'm doing it, I can't find a combination like
   that */
static unsigned char jumpcoas_color_prom[] =
{
	/* palette red component*/
	0x00,0x00,0x0c,0x0f,0x0a,0x0f,0x0b,0x0b,0x00,0x00,0x0d,0x00,0x0d,0x0f,0x04,0x0a,
	0x00,0x0f,0x05,0x0d,0x0c,0x09,0x09,0x0a,0x00,0x09,0x0b,0x00,0x00,0x00,0x0d,0x09,
	0x00,0x00,0x00,0x0e,0x07,0x0f,0x0e,0x00,0x00,0x0b,0x0d,0x0e,0x0c,0x0b,0x0f,0x00,
	0x00,0x0f,0x0a,0x0d,0x00,0x08,0x0f,0x00,0x00,0x0e,0x09,0x0d,0x07,0x03,0x0f,0x0f,
	0x00,0x07,0x0c,0x0f,0x05,0x0f,0x05,0x05,0x00,0x00,0x0d,0x00,0x0d,0x0f,0x04,0x0a,
	0x00,0x0f,0x05,0x0d,0x0c,0x09,0x09,0x0a,0x00,0x09,0x0b,0x00,0x00,0x00,0x0d,0x09,
	0x00,0x00,0x00,0x0e,0x07,0x0f,0x0e,0x00,0x00,0x0b,0x0d,0x0e,0x0c,0x0b,0x0f,0x00,
	0x00,0x0f,0x0a,0x0d,0x00,0x08,0x0f,0x00,0x00,0x0e,0x09,0x0d,0x07,0x03,0x0f,0x0f,
	0x00,0x00,0x0c,0x0f,0x0a,0x0f,0x0c,0x0f,0x00,0x00,0x0d,0x00,0x0d,0x0f,0x04,0x0a,
	0x00,0x0f,0x05,0x0d,0x0c,0x09,0x09,0x0a,0x00,0x09,0x0b,0x00,0x00,0x00,0x0d,0x09,
	0x00,0x00,0x00,0x0e,0x07,0x0f,0x0e,0x00,0x00,0x0b,0x0d,0x0e,0x0c,0x0b,0x0f,0x00,
	0x00,0x0f,0x0a,0x0d,0x00,0x08,0x0f,0x00,0x00,0x0e,0x09,0x0d,0x07,0x03,0x0f,0x0f,
	0x00,0x00,0x0c,0x0f,0x07,0x0f,0x08,0x0b,0x00,0x00,0x0d,0x00,0x0d,0x0f,0x04,0x0a,
	0x00,0x0f,0x05,0x0d,0x0c,0x09,0x09,0x0a,0x00,0x09,0x0b,0x00,0x00,0x00,0x0d,0x09,
	0x00,0x00,0x00,0x0e,0x07,0x0f,0x0e,0x00,0x00,0x0b,0x0d,0x0e,0x0c,0x0b,0x0f,0x00,
	0x00,0x0f,0x0a,0x0d,0x00,0x08,0x0f,0x00,0x00,0x0e,0x09,0x0d,0x07,0x03,0x0f,0x0f,

	/* palette green component*/
	0x00,0x0d,0x0c,0x00,0x05,0x0f,0x07,0x0b,0x00,0x0b,0x07,0x00,0x0b,0x0f,0x04,0x07,
	0x00,0x0f,0x04,0x0b,0x0c,0x05,0x09,0x04,0x00,0x00,0x04,0x00,0x0f,0x00,0x00,0x06,
	0x00,0x06,0x00,0x0d,0x00,0x0f,0x00,0x0f,0x00,0x09,0x0c,0x0c,0x0f,0x0a,0x0f,0x08,
	0x00,0x00,0x07,0x0b,0x0b,0x00,0x0f,0x00,0x00,0x0c,0x07,0x09,0x05,0x03,0x0f,0x00,
	0x00,0x0d,0x0c,0x05,0x06,0x0f,0x05,0x0c,0x00,0x0b,0x07,0x00,0x0b,0x0f,0x04,0x07,
	0x00,0x0f,0x04,0x0b,0x0c,0x05,0x09,0x04,0x00,0x00,0x04,0x00,0x0f,0x00,0x00,0x06,
	0x00,0x06,0x00,0x0d,0x00,0x0f,0x00,0x0f,0x00,0x09,0x0c,0x0c,0x0f,0x0a,0x0f,0x08,
	0x00,0x00,0x07,0x0b,0x0b,0x00,0x0f,0x00,0x00,0x0c,0x07,0x09,0x05,0x03,0x0f,0x00,
	0x00,0x0f,0x0c,0x00,0x05,0x0f,0x07,0x0b,0x00,0x0b,0x07,0x00,0x0b,0x0f,0x04,0x0f,
	0x00,0x0f,0x04,0x0b,0x0c,0x05,0x09,0x04,0x00,0x00,0x04,0x00,0x0f,0x00,0x00,0x06,
	0x00,0x06,0x00,0x0d,0x00,0x0f,0x00,0x0f,0x00,0x09,0x0c,0x0c,0x0f,0x0a,0x0f,0x08,
	0x00,0x00,0x07,0x0b,0x0b,0x00,0x0f,0x00,0x00,0x0c,0x07,0x09,0x05,0x03,0x0f,0x00,
	0x00,0x0d,0x0c,0x00,0x07,0x0f,0x08,0x0b,0x00,0x0b,0x07,0x00,0x0b,0x0f,0x04,0x07,
	0x00,0x0f,0x04,0x0b,0x0c,0x05,0x09,0x04,0x00,0x00,0x04,0x00,0x0f,0x00,0x00,0x06,
	0x00,0x06,0x00,0x0d,0x00,0x0f,0x00,0x0f,0x00,0x09,0x0c,0x0c,0x0f,0x0a,0x0f,0x08,
	0x00,0x00,0x07,0x0b,0x0b,0x00,0x0f,0x00,0x00,0x0c,0x07,0x09,0x05,0x03,0x0f,0x00,

	/* palette blue component*/
	0x00,0x00,0x0c,0x00,0x00,0x0f,0x00,0x00,0x00,0x00,0x00,0x0c,0x00,0x00,0x04,0x00,
	0x00,0x00,0x03,0x06,0x0f,0x05,0x0c,0x04,0x00,0x00,0x04,0x0f,0x00,0x00,0x00,0x00,
	0x00,0x00,0x09,0x00,0x00,0x0f,0x00,0x0f,0x00,0x00,0x06,0x00,0x00,0x07,0x00,0x08,
	0x00,0x00,0x00,0x00,0x00,0x08,0x0f,0x0f,0x00,0x00,0x00,0x00,0x03,0x0d,0x0f,0x00,
	0x00,0x07,0x0c,0x05,0x0a,0x0f,0x0d,0x0d,0x00,0x00,0x00,0x0c,0x00,0x00,0x04,0x00,
	0x00,0x00,0x03,0x06,0x0f,0x05,0x0c,0x04,0x00,0x00,0x04,0x0f,0x00,0x00,0x00,0x00,
	0x00,0x00,0x09,0x00,0x00,0x0f,0x00,0x0f,0x00,0x00,0x06,0x00,0x00,0x07,0x00,0x08,
	0x00,0x00,0x00,0x00,0x00,0x08,0x0f,0x0f,0x00,0x00,0x00,0x00,0x03,0x03,0x0f,0x00,
	0x00,0x0a,0x0c,0x00,0x08,0x0f,0x0a,0x0d,0x00,0x00,0x00,0x0c,0x00,0x00,0x04,0x00,
	0x00,0x00,0x03,0x06,0x0f,0x05,0x0c,0x04,0x00,0x00,0x04,0x0f,0x00,0x00,0x00,0x00,
	0x00,0x00,0x09,0x00,0x00,0x0f,0x00,0x0f,0x00,0x00,0x06,0x00,0x00,0x07,0x00,0x08,
	0x00,0x00,0x00,0x00,0x00,0x08,0x0f,0x0f,0x00,0x00,0x00,0x00,0x03,0x03,0x0f,0x00,
	0x00,0x00,0x0c,0x00,0x09,0x0f,0x0a,0x0e,0x00,0x00,0x00,0x0c,0x00,0x00,0x04,0x00,
	0x00,0x00,0x03,0x06,0x0f,0x05,0x0c,0x04,0x00,0x00,0x04,0x0f,0x00,0x00,0x00,0x00,
	0x00,0x00,0x09,0x00,0x00,0x0f,0x00,0x0f,0x00,0x00,0x06,0x00,0x00,0x07,0x00,0x08,
	0x00,0x00,0x00,0x00,0x00,0x08,0x0f,0x0f,0x00,0x00,0x00,0x00,0x03,0x03,0x0f,0x00,
};



static struct AY8910interface ay8910_interface =
{
	1,      /* 1 chip */
	1000000,	/* 1.0 MHz ? */
	{ 0x20ff },
	{ 0 },
	{ 0 },
	{ 0 },
	{ 0 }
};



static struct MachineDriver machine_driver =
{
	/* basic machine hardware */
	{
		{
			CPU_Z80,
			18432000/6,	/* 3.072 Mhz */
			0,
			readmem,writemem,0,0,
			nmi_interrupt,1
		}
	},
	60, 2500,	/* frames per second, vblank duration */
	1,	/* single CPU, no need for interleaving */
	0,

	/* video hardware */
	32*8, 32*8, { 0*8, 32*8-1, 2*8, 30*8-1 },
	gfxdecodeinfo,
	256,32*8,
	jumpcoas_vh_convert_color_prom,

	VIDEO_TYPE_RASTER,
	0,
	galaxian_vh_start,
	generic_vh_stop,
	galaxian_vh_screenrefresh,

	/* sound hardware */
	0,0,0,0,
	{
		{
			SOUND_AY8910,
			&ay8910_interface
		}
	}
};



static int hiload(void)
{
	/* Note: the game doesn't seem to initialize the high score display
	   at the top of the screen. Even without high score loading, the
	   screen says that the high score is 0, even though the high score
	   table shows all 5000's */

	unsigned char *RAM = Machine->memory_region[0];

	/* check if the hi score table has already been initialized */
	if (RAM[0xc421] == 0x11)
	{
		void *f;


		if ((f = osd_fopen(Machine->gamedrv->name,0,OSD_FILETYPE_HIGHSCORE,0)) != 0)
		{
			osd_fread(f,&RAM[0xc400],0x45);
			osd_fclose(f);
		}

		return 1;
	}
	else return 0;  /* we can't load the hi scores yet */
}



static void hisave(void)
{
	void *f;
	unsigned char *RAM = Machine->memory_region[0];

	if ((f = osd_fopen(Machine->gamedrv->name,0,OSD_FILETYPE_HIGHSCORE,1)) != 0)
	{
		osd_fwrite(f,&RAM[0xc400],0x45);
		osd_fclose(f);
	}
}


/***************************************************************************

  Game driver(s)

***************************************************************************/

ROM_START( jumpcoas_rom )
	ROM_REGION(0x10000)     /* 64k for code */
	ROM_LOAD( "jumpcoas.001", 0x0000, 0x2000, 0xbffc095a )
	ROM_LOAD( "jumpcoas.002", 0x2000, 0x2000, 0x0f79aebb )
	ROM_LOAD( "jumpcoas.003", 0x4000, 0x2000, 0x8a68ea4e )
	ROM_LOAD( "jumpcoas.004", 0x6000, 0x2000, 0xe82211fe )

	ROM_REGION(0x3000)      /* temporary space for graphics (disposed after conversion) */
	ROM_LOAD( "jumpcoas.007", 0x0000, 0x1000, 0x3bd9e2d1 )
	ROM_LOAD( "jumpcoas.006", 0x1000, 0x1000, 0x9d19f20f )
	ROM_LOAD( "jumpcoas.005", 0x2000, 0x1000, 0x0f56813c )
ROM_END



struct GameDriver jumpcoas_driver =
{
	"Jump Coaster",
	"jumpcoas",
	"Zsolt Vasvari",
	&machine_driver,

	jumpcoas_rom,
	0, 0,
	0,
	0,      /* sound_prom */

	input_ports,

	jumpcoas_color_prom, 0, 0,
	ORIENTATION_ROTATE_90,

	hiload, hisave
};
